import { acceptHMRUpdate, defineStore } from 'pinia'
import {
  AlignEnum,
  ContentLayoutEnum,
  HEADER_PRESET_BG_COLOR_LIST,
  MenuModeEnum,
  MixSidebarTriggerEnum,
  NavBarModeEnum,
  PlacementEnum,
  RouterTransitionEnum,
  SIDE_BAR_BG_COLOR_LIST,
  SessionTimeoutProcessingEnum,
  SettingButtonPositionEnum,
  ThemeEnum,
  TriggerEnum,
} from '~/constants'
import { _assign, _merge } from '~/utils'

const initState: DefineAppConfigOptions = {
  theme: ThemeEnum.LIGHT,
  navBarMode: NavBarModeEnum.SIDEBAR,
  themeColor: '',
  showThemeModeToggle: true,
  openKeepAlive: true,
  useOpenBackTop: true,
  closeMessageOnSwitch: false,
  removeAllHttpPending: true,
  // permissionCacheType: CacheTypeEnum.LOCAL,
  settingButtonPosition: SettingButtonPositionEnum.AUTO,
  openSettingDrawer: false,
  // permissionMode: PermissionModeEnum.ROUTE_MAPPING,
  sessionTimeoutProcessing: SessionTimeoutProcessingEnum.ROUTE_JUMP,
  grayMode: false,
  colorWeak: false,
  lockTime: 0,
  useLockPage: false,
  canEmbedIFramePage: true,
  closeMixSidebarOnChange: false,
  sidebar: {
    theme: ThemeEnum.LIGHT,
    show: true,
    visible: true,
    bgColor: SIDE_BAR_BG_COLOR_LIST[0],
    fixed: false,
    width: 210,
    mixSidebarWidth: 80,
    collapsedWidth: 48,
    collapsed: false,
    trigger: TriggerEnum.CENTER,
  },
  menu: {
    show: true,
    canDrag: false,
    split: false,
    mode: MenuModeEnum.VERTICAL,
    accordion: false,
    collapsedShowTitle: false,
    mixSideTrigger: MixSidebarTriggerEnum.CLICK,
    mixSideFixed: false,
    topMenuAlign: AlignEnum.START,
    dropdownPlacement: PlacementEnum.TOP_START,
    subMenuWidth: 0,
  },
  header: {
    theme: ThemeEnum.DARK,
    show: true,
    visible: true,
    bgColor: HEADER_PRESET_BG_COLOR_LIST[0],
    fixed: false,
    height: 48,
    showDoc: true,
    showBreadCrumb: true,
    showBreadCrumbIcon: true,
    showFullScreen: true,
    showNotice: true,
    showSearch: true,
    showLocalePicker: true,
    showSetting: true,
  },
  logo: {
    show: true,
    visible: true,
    showTitle: true,
  },
  tabTar: {
    show: true,
    visible: true,
    height: 36,
    cache: true,
    canDrag: false,
    showFold: true,
    showQuick: true,
    showRedo: true,
  },
  content: {
    fullScreen: false,
    mode: ContentLayoutEnum.FULL,
  },
  footer: {
    height: 60,
    show: false,
    visible: false,
  },
  transition: {
    enable: true,
    basicTransition: RouterTransitionEnum.FADE_SIDE,
    openPageLoading: true,
    openNProgress: false,
  },
}

export const useAppConfigStore = defineStore('APP_CONFIG', () => {
  let state = $ref<DefineAppConfigOptions>(Object.assign({}, initState))

  // computed
  const isSidebar = computed(() => state.navBarMode === NavBarModeEnum.SIDEBAR)
  const isTopMenu = computed(() => state.navBarMode === NavBarModeEnum.TOP_MENU)
  const isMixSidebar = computed(() => state.navBarMode === NavBarModeEnum.MIX_SIDEBAR)
  const isMix = computed(() => state.navBarMode === NavBarModeEnum.MIX)
  const isMixMode = computed(() => state.navBarMode === NavBarModeEnum.MIX && state.menu.mode === MenuModeEnum.INLINE)
  const isHorizontal = computed(() => state.menu.mode === MenuModeEnum.HORIZONTAL)
  const tagTarCache = computed(() => state.tabTar.cache)

  const isCollapsedShowTitle = computed(() => {
    if (unref(isMixSidebar) || unref(isSidebar))
      return state.sidebar.collapsed

    return state.menu.collapsedShowTitle && state.sidebar.collapsed
  })

  function $reset() {
    state = Object.assign({}, initState)
  }

  function setAppConfig(configs: DeepPartial<DefineAppConfigOptions>) {
    state = _merge(state, configs)
  }

  function setSidebar(value: Partial<Omit<SidebarConfigOptions, 'mixSidebarWidth' | 'collapsedWidth'>>) {
    _assign(state.sidebar, value)
  }

  function setMenu(value: Partial<MenuConfigOptions>) {
    _assign(state.menu, value)
  }

  function setHeader(value: Partial<Omit<HeaderConfigOptions, 'height'>>) {
    _assign(state.header, value)
  }

  function setLogo(value: Partial<LogoConfigOptions>) {
    _assign(state.logo, value)
  }

  function setTabBar(value: Partial<TabTarConfigOptions>) {
    _assign(state.tabTar, value)
  }

  function setContent(value: Partial<ContentConfigOptions>) {
    _assign(state.content, value)
  }

  function setFooter(value: Partial<Omit<FooterConfigOptions, 'height'>>) {
    _assign(state.footer, value)
  }

  function setTransition(value: Partial<TransitionConfigOptions>) {
    _assign(state.transition, value)
  }

  return {
    $state: state,
    ...toRefs(state),
    isSidebar,
    isTopMenu,
    isMixSidebar,
    isMix,
    isMixMode,
    isHorizontal,
    isCollapsedShowTitle,
    tagTarCache,
    setAppConfig,
    $reset,
    // setter
    setSidebar,
    setMenu,
    setHeader,
    setLogo,
    setTabBar,
    setContent,
    setFooter,
    setTransition,
  }
}, {
  persist: {
    paths: Object.keys(initState),
  },
})

if (import.meta.hot)
  import.meta.hot.accept(acceptHMRUpdate(useAppConfigStore as any, import.meta.hot))
